// PROM Programmer
//   A factory PROM is Low and the Programmer write Highs.
//   This code programs the low nibble.
//   ROM is uploaded to Program Memory.
//   Connect the programmer to the Arduino and then power up the Arduino.
//   The programmer has pull up resistors on Vpp and CS.
//   After the PROM has been programmed the LED will blink slowly if successful (1s on/1s off).
//   If if unsuccessful the the LED will blink quickly (0.1s on/0.1s off).

#include <Arduino.h>
#include <avr/pgmspace.h>
const int bytes=128;

void setup(){
  // Initialise ports

  // But we will do the same for the Arduino
  // Bit   76543210
  // PortD VAAAAAAA
  //       P6543210
  PORTD = B10000000; // Turn on pull ups for Vpp (active low)

  // PortB xxxCDDDD
  //          S3210
  PORTB = B00010000; // Turn on pull ups for CS (active low)

  DDRD  = B11111111; // Make Port D an output
  PORTD = B10000000; // Make Vpp (active low) HIGH

  DDRB  = B00111111; // Make Port B an output
  PORTB = B00010000; // Make CS (active low) HIGH
                     // Note LED is off
  // PortC xxxxAAAA
  //           3210
  DDRC  = B00000000; // Make Port C an input
  PORTC = B00000000; // No pull ups
}

// Here is the PROM Code
const byte PROGMEM ROM[bytes] ={
// Monitor
  0X7F, 0XF2,  //  S0: 0X00  _128 -> ADDR
  0XF2, 0XFA,  //  L0: 0X02  ADDR -> A
  0X7E, 0XF7,  //      0X04  _255 -> JUMP
  0X7D, 0XF6,  //      0X06  _241 -> RPTR
  0X7C, 0XF5,  //      0X08  _243 -> DEPOSIT
  0XF2, 0XF4,  //      0X0A  ADDR -> FETCH
  0X77, 0XF1,  //      0X0C  L1 -> RETURN
  0X7B, 0XFF,  //      0X0E  _244 -> JMP
  0XF3, 0XFB,  //  L1: 0X10  DATA -> D
  0XFA, 0XF8,  //      0X12  A -> P
  0XFA, 0XF9,  //      0X14  A -> Q
  0XF9, 0XF8,  //      0X16  NAND -> P
  0XF2, 0XF9,  //      0X18  ADDR -> Q
  0XF9, 0XF9,  //      0X1A  NAND -> Q
  0XF9, 0XF0,  //      0X1C  NAND -> TMP
  0XF2, 0XF8,  //      0X1E  ADDR -> P
  0XF9, 0XF9,  //      0X20  NAND -> Q
  0XF0, 0XF8,  //      0X22  TMP -> P
  0XF9, 0XF2,  //      0X24  NAND -> ADDR
  0XFB, 0XF8,  //      0X26  D -> P
  0XFB, 0XF9,  //      0X28  D -> Q
  0XF9, 0XF8,  //      0X2A  NAND -> P
  0XF3, 0XF9,  //      0X2C  DATA -> Q
  0XF9, 0XF9,  //      0X2E  NAND -> Q
  0XF9, 0XF0,  //      0X30  NAND -> TMP
  0XF3, 0XF8,  //      0X32  DATA -> P
  0XF9, 0XF9,  //      0X34  NAND -> Q
  0XF0, 0XF8,  //      0X36  TMP -> P
  0XF9, 0XF3,  //      0X38  NAND -> DATA
  0XFB, 0XF8,  //      0X3A  D -> P
  0X7A, 0XF9,  //      0X3C  _1 -> Q
  0X76, 0XFE,  //      0X3E  L2 -> JC
  0XF2, 0XF8,  //      0X40  ADDR -> P
  0X7A, 0XF9,  //      0X42  _1 -> Q
  0X7F, 0XFE,  //      0X44  _128 -> JC
  0X7E, 0XF7,  //      0X46  _255 -> JUMP
  0X7D, 0XF6,  //      0X48  _241 -> RPTR
  0XF2, 0XF5,  //      0X4A  ADDR -> DEPOSIT
  0X7C, 0XF4,  //      0X4C  _243 -> FETCH
  0X76, 0XF1,  //      0X4E  L2 -> RETURN
  0X7B, 0XFF,  //      0X50  _244 -> JMP
  0XFA, 0XF8,  //  L2: 0X52  A -> P
  0XFB, 0XF9,  //      0X54  D -> Q
  0XF9, 0XF8,  //      0X56  NAND -> P
  0X7E, 0XF9,  //      0X58  _255 -> Q
  0X76, 0XFE,  //      0X5A  L2 -> JC
  0X75, 0XF1,  //  L3: 0X5C  L0 -> RETURN
  0X79, 0XF0,  //      0X5E  _10 -> DELAY
  0XF0, 0XF8,  //  D0: 0X60  DELAY -> P
  0X7E, 0XF9,  //      0X62  _255 -> Q
  0X74, 0XFE,  //      0X64  D1 -> JC
  0XF1, 0XFF,  //      0X66  RETURN -> JMP
  0XF8, 0XF0,  //  D1: 0X68  ADD -> DELAY
  0X78, 0XF8,  //      0X6A  _123 -> P
  0XF8, 0XF8,  //  D2: 0X6C  ADD -> P
  0X73, 0XFE,  //      0X6E  D2 -> JC
  0X72, 0XFF,  //      0X70  D0 -> JMP
  0X60, 0X6C,  //      0X72  Const,   Const
  0X68, 0X02,  //      0X74  Const,   Const
  0X52, 0X10,  //      0X76  Const,   Const
  0X7B, 0X0A,  //      0X78  Const,   Const
  0X01, 0XF4,  //      0X7A  Const,   Const
  0XF3, 0XF1,  //      0X7C  Const,   Const
  0XFF, 0X80   //      0X7E  Const,   Const
};

byte CPort;
byte Data;
byte Addr,BitNo,Pass;
int  Delay;
byte Verify;
void loop(){
  // Make sure controls are set
  PORTD=B10000000; // Vpp HIGH (voltage normal or 5V)
  PORTB=B00010000; // CS HIGH (unselected)
  
  for (Addr=0;Addr<bytes;Addr++) {
    // Read the ROM
    Data=pgm_read_byte(ROM+Addr);
    // Set Address (make sure Vpp is HIGH)
    PORTD=(B10000000|Addr);
    delayMicroseconds(1);
    for (BitNo=0;BitNo<4;BitNo++) {
      if (((Data>>BitNo)&B00000001) == 1) {
        // Set Data bit (make sure CS is HIGH)
        PORTB=B00010000|(B00000001<<BitNo);
        delayMicroseconds(1);
        // Write bit cycle
        for (Verify=0;Verify<2;Verify++) {
          // Write the bit
          for (Pass=0;Pass<10;Pass++) {
            PORTD=PORTD&B01111111; // Bring up Vpp to 10.5v (active low)
            delayMicroseconds(1);
            PORTB=PORTB&B00001111; // Select chip (active low)
            delayMicroseconds(10);
            PORTB=PORTB|B00010000; // Unselect chip
            PORTD=PORTD|B10000000; // Bring down Vpp to 5v
            delayMicroseconds(33); // Maximum duty cycle is 25%
          }

          // Verify
          PORTB=B00000000;         // Make CS and Data In 0-3 low
          delayMicroseconds(10);   // Wait for high to low transition
          // Read the PROM
          CPort=PINC;
          if ((Data&(B00000001<<BitNo))==(CPort&(B00000001<<BitNo))) break;

        }
        
        // Post verify bit write
        PORTB=B00010000|(B00000001<<BitNo); // Set Data bit (make sure CS is HIGH)
        for (Pass=0;Pass<5;Pass++) {
          PORTD=PORTD&B01111111; // Bring up Vpp to 10.5v (active low)
          delayMicroseconds(1);
          PORTB=PORTB&B00001111; // Select chip (active low)
          delayMicroseconds(10);
          PORTB=PORTB|B00010000; // Unselect chip
          PORTD=PORTD|B10000000; // Bring down Vpp to 5v
          delayMicroseconds(33); // Maximum duty cycle is 25%
        }        
      }
    }
  }
  
  // Check read prom (low nibble)
  Delay=1000;
  // Make sure controls are set
  PORTB=B00000000; // Make CS and Data In 0-3 low
  for (Addr=0;Addr<bytes;Addr++) {
    Data=pgm_read_byte(ROM+Addr);
    // Set Address (make sure Vpp is LOW (normal 5v))
    PORTD=(B10000000|Addr);
    delayMicroseconds(2); // Wait for high to low transition
    // Read the ROM
    CPort=PINC;
    if ((Data&B00001111)!=(CPort&B00001111)) {
      Delay=100;
    }
  }
  // PORTB=B00010000; // Make CS HIGH (unselect)
  // PORTD=B10000000; // Set Vpp HIGH (normal 5v)
  
  while(true){
    digitalWrite(13,HIGH);
    delay(Delay);
    digitalWrite(13,LOW);
    delay(Delay);
  };

}


